// list1.h
// Glenn G. Chappell
// 19 Nov 2013
//
// For CS 202 Fall 2013
// Header for class template List1
// Singly Linked List node class
// There is no associated source file

#ifndef FILE_LIST1_H_INCLUDED
#define FILE_LIST1_H_INCLUDED

#include <algorithm>
using std::swap;


// class List1
// Linked-list node holding value, owning remainder of list.
// Includes creation, copying, destruction operations.
template<typename ValueType>
class List1 {

// ***** List1: ctors, dctor, op= *****
public:

    // Ctor (default, from value, from value & ptr)
    // Parameter next, if given, should either be nullptr or ptr to
    // valid List1, allocated with new, ownership transfered to *this.
    // If not given, next is set to nullptr, marking end of list.
    List1(const ValueType & val=ValueType(),
          List1 * next=nullptr)
        :_val(val),
         _next(next)
    {}

    // Copy ctor
    // Copy all nodes in list.
    List1(const List1 & other)
    {
        _val = other._val;
        if (other._next == nullptr)
        {
            _next = other._next;
        }
        else
        {
            _next = new List1(*other._next);
        }
    }

    // operator= (from List1)
    // Copy all nodes in list.
    List1 & operator=(const List1 & rhs)
    {
        // Use the swap trick
        List1 copy(rhs);
        swap(_val, copy._val);
        swap(_next, copy._next);
        return *this;
    }

    // Dctor
    // Destroy all nodes in list.
    ~List1()
    {
         delete _next;
    }

// ***** List1: general public functions *****
public:

    // Get- & set-functions for our value
    ValueType getValue() const
    {
        return _val;
    }
    void setValue(const ValueType & val)
    {
        _val = val;
    }

    // Get function for our next-node pointer.
    List1 * getNext() const
    {
        return _next;
    }
    // No set function for next-node pointer,
    //  since the idea gives me the willies. -GGC-

// ***** List1: member variables *****
private:

    ValueType _val;  // Our value
    List1 * _next;   // Next-node pointer
                     //  Should either be nullptr, marking end of list,
                     //  OR ptr to valid List1, allocated with new,
                     //  owned by *this.

};


#endif //#ifndef FILE_LIST1_H_INCLUDED

